@@ -45,7 +45,7 @@ static char sccsid[] = "@(#)ftp.c	8.6 (Berkeley) 10/27/94";
 #include <netinet/in.h>
 #include <netinet/in_systm.h>
 #include <netinet/ip.h>
-#ifdef INET6
+#if INET6
 #include <netinet6/in6.h>
 #endif /* INET6 */
 #include <arpa/inet.h>
@@ -71,13 +71,13 @@ static char sccsid[] = "@(#)ftp.c	8.6 (Berkeley) 10/27/94";
 #include <setjmp.h>
 #endif /* FASTCTO */
 
-#ifdef INET6
+#if INET6
 #include "support.h"
 #endif /* INET6 */
 
 extern int h_errno;
 
-#ifdef INET6
+#if INET6
 #define su union { \
   struct sockaddr sa; \
   struct sockaddr_in sin; \
@@ -107,6 +107,11 @@ FILE	*cin, *cout;
 typedef void (*sig_t)(int);
 #endif /* linux */
 
+#if IPSEC
+extern void *control_request, *data_request;
+extern int control_requestlen, data_requestlen;
+#endif /* IPSEC */
+
 #ifdef FASTCTO
 static jmp_buf timeout_env;
 
@@ -119,13 +124,13 @@ static void timeout_handler(int i)
 char *
 hookup(host, port)
 	char *host;
-#ifdef INET6
+#if INET6
 	char *port;
 #else /* INET6 */
 	int port;
 #endif /* INET6 */
 {
-#ifdef INET6
+#if INET6
   int i;
   struct addrinfo req, *ai, *ai2;
   int s, len, tos;
@@ -165,6 +170,12 @@ hookup(host, port)
       continue;
     }
     
+#if IPSEC
+    if (control_request)
+      if (setsockopt(s, SOL_SOCKET, SO_SECURITY_REQUEST, control_request, control_requestlen) < 0)
+        fprintf(stderr, "ftp: setsockopt(..., SO_SECURITY_REQUEST, control_request, ...): %s(%d)\n", strerror(errno), errno);
+#endif /* IPSEC */
+
 #if FASTCTO
     if (setjmp(timeout_env)) {
       fprintf(stderr, "ftp: Connection timed out\n");
@@ -179,6 +190,7 @@ hookup(host, port)
       alarm(0);
 #endif /* FASTCTO */
       perror("ftp: connect");
+      errno = 0;
       close(s);
       s = -1;
       continue;
@@ -265,13 +277,13 @@ hookup(host, port)
 		goto bad;
 	}
 #ifdef IP_TOS
-#ifdef INET6
+#if INET6
 	if (myctladdr.sa.sa_family == AF_INET) {
 #endif /* INET6 */
 	tos = IPTOS_LOWDELAY;
 	if (setsockopt(s, IPPROTO_IP, IP_TOS, (char *)&tos, sizeof(int)) < 0)
 		warn("setsockopt TOS (ignored)");
-#ifdef INET6
+#if INET6
 	}
 #endif /* INET6 */
 #endif
@@ -1131,7 +1143,7 @@ initconn()
 #endif /* INET6 */
 
 	if (passivemode) {
-#ifdef INET6
+#if INET6
 		data = socket(hisctladdr.sa.sa_family, SOCK_STREAM, 0);
 #else /* INET6 */
 		data = socket(AF_INET, SOCK_STREAM, 0);
@@ -1144,7 +1156,12 @@ initconn()
 		    setsockopt(data, SOL_SOCKET, SO_DEBUG, (char *)&on,
 			       sizeof (on)) < 0)
 			perror("ftp: setsockopt (ignored)");
-#ifdef INET6
+#if IPSEC
+                if (data_request)
+		  if (setsockopt(data, SOL_SOCKET, SO_SECURITY_REQUEST, data_request, data_requestlen) < 0)
+		    fprintf(stderr, "ftp: setsockopt(..., SO_SECURITY_REQUEST, data_request, ...): %s(%d)\n", strerror(errno), errno);
+#endif /* IPSEC */
+#if INET6
 		switch (howport) {
 		  case 0:
 		  case 1:
@@ -1220,14 +1237,14 @@ initconn()
 		}
 #endif /* INET6 */
 #ifdef IP_TOS
-#ifdef INET6
+#if INET6
 		if (data_addr.sa.sa_family == AF_INET) {
 #endif /* INET6 */
 		on = IPTOS_THROUGHPUT;
 		if (setsockopt(data, IPPROTO_IP, IP_TOS, (char *)&on,
 			       sizeof(int)) < 0)
 			perror("ftp: setsockopt TOS (ignored)");
-#ifdef INET6
+#if INET6
 		}
 #endif /* INET6 */
 #endif
@@ -1235,7 +1252,7 @@ initconn()
 	}
 
 noport:
-#ifdef INET6
+#if INET6
   memcpy(&data_addr, &myctladdr, NRL_SA_LEN((struct sockaddr *)&myctladdr));
   if (sendport)
      switch(myctladdr.sa.sa_family) {
@@ -1266,12 +1283,17 @@ noport:
 			sendport = 1;
 		return (1);
 	}
+#if IPSEC
+        if (data_request)
+	  if (setsockopt(data, SOL_SOCKET, SO_SECURITY_REQUEST, data_request, data_requestlen) < 0)
+	    fprintf(stderr, "ftp: setsockopt(..., SO_SECURITY_REQUEST, data_request, ...): %s(%d)\n", strerror(errno), errno);
+#endif /* IPSEC */
 	if (!sendport)
 		if (setsockopt(data, SOL_SOCKET, SO_REUSEADDR, (char *)&on, sizeof (on)) < 0) {
 			warn("setsockopt (reuse address)");
 			goto bad;
 		}
-#ifdef INET6
+#if INET6
 	if (bind(data, (struct sockaddr *)&data_addr, NRL_SA_LEN((struct sockaddr *)&data_addr)) < 0) {
 #else /* INET6 */
 	if (bind(data, (struct sockaddr *)&data_addr, sizeof (data_addr)) < 0) {
@@ -1290,7 +1312,7 @@ noport:
 	if (listen(data, 1) < 0)
 		warn("listen");
 	if (sendport) {
-#ifdef INET6
+#if INET6
 	  char buffer[80];
 
 	  switch (howport) {
@@ -1365,13 +1387,13 @@ noport:
 	if (tmpno)
 		sendport = 1;
 #ifdef IP_TOS
-#ifdef INET6
+#if INET6
 	if (data_addr.sa.sa_family == AF_INET) {
 #endif /* INET6 */
 	on = IPTOS_THROUGHPUT;
 	if (setsockopt(data, IPPROTO_IP, IP_TOS, (char *)&on, sizeof(int)) < 0)
 		warn("setsockopt TOS (ignored)");
-#ifdef INET6
+#if INET6
 	}
 #endif /* INET6 */
 #endif
@@ -1387,7 +1409,7 @@ FILE *
 dataconn(lmode)
 	char *lmode;
 {
-#ifdef INET6
+#if INET6
   su from;
 #else /* INET6 */
 	struct sockaddr_in from;
@@ -1406,13 +1428,13 @@ dataconn(lmode)
 	(void) close(data);
 	data = s;
 #ifdef IP_TOS
-#ifdef INET6
+#if INET6
 	if (from.sa.sa_family == AF_INET) {
 #endif /* INET6 */
 	tos = IPTOS_THROUGHPUT;
 	if (setsockopt(s, IPPROTO_IP, IP_TOS, (char *)&tos, sizeof(int)) < 0)
 		warn("setsockopt TOS (ignored)");
-#ifdef INET6
+#if INET6
 	}
 #endif /* INET6 */
 #endif
@@ -1478,7 +1500,7 @@ pswitch(flag)
 	static struct comvars {
 		int connect;
 		char name[MAXHOSTNAMELEN];
-#ifdef INET6
+#if INET6
 	        su mctl;
 	        su hctl;
 #else /* INET6 */
@@ -1525,7 +1547,7 @@ pswitch(flag)
 	} else
 		ip->name[0] = 0;
 	hostname = op->name;
-#ifdef INET6
+#if INET6
 	memcpy(&ip->hctl, &hisctladdr, NRL_SA_LEN((struct sockaddr *)&hisctladdr));
 	memcpy(&hisctladdr, &op->hctl, NRL_SA_LEN((struct sockaddr *)&op->hctl));
 	memcpy(&ip->mctl, &myctladdr, NRL_SA_LEN((struct sockaddr *)&myctladdr));
