@@ -206,6 +206,20 @@ static struct passwd *
 		 sgetpwnam __P((char *));
 static char	*sgetsave __P((char *));
 
+void enable_signalling()
+{
+  signal(SIGPIPE, lostconn);
+  if ((int)signal(SIGURG, myoob) < 0)
+    syslog(LOG_ERR, "signal: %m");
+};
+
+void disable_signalling()
+{
+  signal(SIGPIPE, SIG_IGN);
+  if ((int)signal(SIGURG, SIG_IGN) < 0)
+    syslog(LOG_ERR, "signal: %m");
+};
+
 static char *
 curdir()
 {
@@ -350,10 +364,8 @@ main(argc, argv, envp)
 		}
 	}
 	(void) freopen(_PATH_DEVNULL, "w", stderr);
-	(void) signal(SIGPIPE, lostconn);
 	(void) signal(SIGCHLD, SIG_IGN);
-	if ((int)signal(SIGURG, myoob) < 0)
-		syslog(LOG_ERR, "signal: %m");
+        enable_signalling();
 
 	/* Try to handle urgent data inline */
 #ifdef SO_OOBINLINE
@@ -592,13 +604,14 @@ checkuser(name)
 static void
 end_login()
 {
-
+	disable_signalling();
 	(void) seteuid((uid_t)0);
 	if (logged_in)
 		logwtmp(ttyline, "", "");
 	pw = NULL;
 	logged_in = 0;
 	guest = 0;
+	enable_signalling();
 }
 
 void
@@ -671,6 +684,7 @@ pass(passwd)
 		reply(550, "Can't set uid.");
 		goto bad;
 	}
+	enable_signalling();
 	/*
 	 * Display a login message, if it exists.
 	 * N.B. reply(230,) must follow the message.
@@ -862,6 +876,7 @@ getdatasock(mode)
 
 	if (data >= 0)
 		return (fdopen(data, mode));
+	disable_signalling();
 	(void) seteuid((uid_t)0);
 #if INET6
 	s = socket(ctrl_addr.sa.sa_family, SOCK_STREAM, 0);
@@ -897,6 +912,7 @@ getdatasock(mode)
 		sleep(tries);
 	}
 	(void) seteuid((uid_t)pw->pw_uid);
+	enable_signalling();
 #ifdef IP_TOS
 #if INET6
 	if (data_source.sa.sa_family == AF_INET) {
@@ -913,6 +929,7 @@ bad:
 	/* Return the real value of errno (close may change it) */
 	t = errno;
 	(void) seteuid((uid_t)pw->pw_uid);
+	enable_signalling();
 	(void) close(s);
 	errno = t;
 	return (NULL);
@@ -1525,7 +1542,7 @@ void
 dologout(status)
 	int status;
 {
-
+	disable_signalling();
 	if (logged_in) {
 		(void) seteuid((uid_t)0);
 		logwtmp(ttyline, "", "");
@@ -1612,6 +1629,7 @@ passive()
 	pasv_addr = ctrl_addr;
 	pasv_addr.sin_port = 0;
 #endif /* INET6 */
+	disable_signalling();
 	(void) seteuid((uid_t)0);
 #if INET6
 	if (bind(pdata, (struct sockaddr *)&pasv_addr, NRL_SA_LEN((struct sockaddr *)&pasv_addr)) < 0) {
@@ -1619,9 +1637,11 @@ passive()
 	if (bind(pdata, (struct sockaddr *)&pasv_addr, sizeof(pasv_addr)) < 0) {
 #endif /* INET6 */
 		(void) seteuid((uid_t)pw->pw_uid);
+		enable_signalling();
 		goto pasv_error;
 	}
 	(void) seteuid((uid_t)pw->pw_uid);
+	enable_signalling();
 	len = sizeof(pasv_addr);
 	if (getsockname(pdata, (struct sockaddr *) &pasv_addr, &len) < 0)
 		goto pasv_error;
