@@ -64,7 +64,7 @@ you didn't get a copy, you may request one from <license@inner.net>.
 
 /*
  *      @(#)netdb.h	8.1 (Berkeley) 6/2/93
- *	$Id: netdb.h,v 1.2 1997/01/12 04:23:09 cmetz Exp $
+ *	$Id: netdb.h,v 1.3 1997/01/18 17:50:14 cmetz Exp $
  */
 
 #ifndef _NETDB_H_
@@ -164,19 +164,12 @@ void		setprotoent __P((int));
 void		setservent __P((int));
 __END_DECLS
 
-/* This is nec'y to make this include file properly replace the sun version. */
-#ifdef sun
-#ifdef __GNU_LIBRARY__
-#include <rpc/netdb.h>
-#else
 struct rpcent {
 	char	*r_name;	/* name of server for this rpc program */
 	char	**r_aliases;	/* alias list */
 	int	r_number;	/* rpc program number */
 };
 struct rpcent	*getrpcbyname(), *getrpcbynumber(), *getrpcent();
-#endif /* __GNU_LIBRARY__ */
-#endif /* sun */
 
 #define AI_PASSIVE      1 /* socket address is intended for bind() */
 #define AI_CANONNAME    2 /* request for canonical name */
