@@ -148,10 +148,6 @@ static char sccsid[] = "@(#)inetd.c	8.4 (Berkeley) 4/13/94";
  *	wait/nowait			single-threaded/multi-threaded
  *	user				user to run daemon as
  *
- *      authentication level            Security levels on listening
- *      ESP transport level             sockets.
- *      ESP network level
- *
  *	server program			full path name
  *	server program arguments	maximum of MAXARGS (20)
  *
@@ -183,6 +179,7 @@ static char sccsid[] = "@(#)inetd.c	8.4 (Berkeley) 4/13/94";
 #include <netinet/in.h>
 #ifdef INET6
 #include <netinet6/in6.h>
+#include <sys/un.h>
 #endif /* INET6 */
 #include <arpa/inet.h>
 
@@ -200,6 +197,10 @@ static char sccsid[] = "@(#)inetd.c	8.4 (Berkeley) 4/13/94";
 #include <syslog.h>
 #include <unistd.h>
 
+#if INET6
+#include "support.h"
+#endif /* INET6 */
+
 #include "pathnames.h"
 
 #define	TOOMANY		40		/* don't start more than TOOMANY */
@@ -217,6 +218,15 @@ int	toomany = TOOMANY;
 struct	servent *sp;
 int nflag = 0;
 
+#if INET6
+typedef union {
+  struct sockaddr sa;
+  struct sockaddr_in sin;
+  struct sockaddr_in6 sin6;
+  struct sockaddr_un sun;
+} su;
+#endif /* INET6 */
+
 struct	servtab {
 	char	*se_service;		/* name of service */
 	int	se_socktype;		/* type of socket to use */
@@ -224,6 +234,10 @@ struct	servtab {
 	short	se_wait;		/* single threaded server */
 	short	se_checked;		/* looked at during merge */
 	char	*se_user;		/* user name to run as */
+#ifdef IPSEC
+        void    *se_request;
+        int     se_requestlen;
+#endif /* IPSEC */
 	struct	biltin *se_bi;		/* if built-in, description */
 	char	*se_server;		/* server program */
 #define	MAXARGV 20
@@ -231,7 +245,7 @@ struct	servtab {
 	int	se_fd;			/* open descriptor */
 	int	se_type;		/* type */
 #ifdef INET6
-	struct	sockaddr_in6 se_ctrladdr;/* bound address */
+        su      se_ctrladdr;
 #else /* INET6 */
 	struct	sockaddr_in se_ctrladdr;/* bound address */
 #endif /* INET6 */
@@ -622,6 +636,22 @@ config(signo)
 			for (i = 0; i < MAXARGV; i++)
 				SWAP(sep->se_argv[i], cp->se_argv[i]);
 
+#ifdef IPSEC
+			if ((cp->se_requestlen != sep->se_requestlen) || (cp->se_request && memcmp(cp->se_request, sep->se_request, cp->se_requestlen))) {
+			  if (sep->se_request)
+			    free(sep->se_request);
+			  sep->se_request = cp->se_request;
+			  sep->se_requestlen = cp->se_requestlen;
+
+			  if (sep->se_request && (sep->se_fd != -1))
+			    if (setsockopt(sep->se_fd, SOL_SOCKET, SO_SECURITY_REQUEST, sep->se_request, sep->se_requestlen) < 0) {
+			      syslog(LOG_ERR, "setsockopt(..., SO_SECURITY_REQUEST, ...): %s(%d)", strerror(errno), errno);
+			      if (debug)
+				fprintf(stderr, "setsockopt(..., SO_SECURITY_REQUEST, ...): %s(%d)\n", strerror(errno), errno);
+			      exit(1);
+			    };
+			};
+#endif /* IPSEC */
 			sigsetmask(omask);
 			freeconfig(cp);
 			if (debug)
@@ -636,6 +666,9 @@ config(signo)
 			sep->se_fd = -1;
 			continue;
 		}
+#if INET6
+
+#else /* INET6 */
 		sp = getservbyname(sep->se_service, sep->se_proto);
 		if (sp == 0) {
 			syslog(LOG_ERR, "%s/%s: unknown service",
@@ -643,23 +676,12 @@ config(signo)
 			sep->se_checked = 0;
 			continue;
 		}
-#ifdef INET6
-		if (sp->s_port != sep->se_ctrladdr.sin6_port) {
-			sep->se_ctrladdr.sin6_family = AF_INET6;
-			sep->se_ctrladdr.sin6_port = sp->s_port;
-#else /* INET6 */
 		if (sp->s_port != sep->se_ctrladdr.sin_port) {
 			sep->se_ctrladdr.sin_family = AF_INET;
 			sep->se_ctrladdr.sin_port = sp->s_port;
-#endif /* INET6 */
 			if (sep->se_fd >= 0)
 				close_sep(sep);
 		}
-#ifdef INET6
-#ifdef SIN6_LEN
-		sep->se_ctrladdr.sin6_len = sizeof (struct sockaddr_in6);
-#endif /* SIN6_LEN */
-#else /* INET6 */
 		sep->se_ctrladdr.sin_len = sizeof (struct sockaddr_in);
 #endif /* INET6 */
 		if (sep->se_fd == -1)
@@ -726,13 +748,36 @@ setsockopt(fd, SOL_SOCKET, opt, (char *)&on, sizeof (on))
 	if (turnon(sep->se_fd, SO_REUSEADDR) < 0)
 		syslog(LOG_ERR, "setsockopt (SO_REUSEADDR): %m");
 #undef turnon
-#if defined(INET6) && OFFSETPORT
-        if (ntohs(sep->se_ctrladdr.sin6_port) < OFFSETPORT)
-          sep->se_ctrladdr.sin6_port=htons(ntohs(sep->se_ctrladdr.sin6_port)+OFFSETPORT);
-#endif /* defined(INET6) && OFFSETPORT */
+#if OFFSETPORT
+	switch(sep->se_ctrladdr.sa.sa_family) {
+	  case AF_INET:
+	    if (ntohs(sep->se_ctrladdr.sin.sin_port) < OFFSETPORT)
+	      sep->se_ctrladdr.sin.sin_port=htons(ntohs(sep->se_ctrladdr.sin.sin_port)+OFFSETPORT);
+	    break;
+	  case AF_INET6:
+	    if (ntohs(sep->se_ctrladdr.sin.sin_port) < OFFSETPORT)
+	      sep->se_ctrladdr.sin.sin_port=htons(ntohs(sep->se_ctrladdr.sin.sin_port)+OFFSETPORT);
+	    break;
+	};
+#endif /* OFFSETPORT */
+
+#ifdef IPSEC
+	if (sep->se_request && (setsockopt(sep->se_fd, SOL_SOCKET, SO_SECURITY_REQUEST, sep->se_request, sep->se_requestlen) < 0)) {
+	  if (setsockopt(sep->se_fd, SOL_SOCKET, SO_SECURITY_REQUEST, sep->se_request, sep->se_requestlen) < 0) {
+	    syslog(LOG_ERR, "setsockopt(..., SO_SECURITY_REQUEST, ...): %s(%d)", strerror(errno), errno);
+	    if (debug)
+	      fprintf(stderr, "setsockopt(..., SO_SECURITY_REQUEST, ...): %s(%d)\n", strerror(errno), errno);
+	    exit(1);
+	  };
+	};
+#endif /* IPSEC */
 
 	if (bind(sep->se_fd, (struct sockaddr *)&sep->se_ctrladdr,
+#if INET6
+	  NRL_SA_LEN((struct sockaddr *)&sep->se_ctrladdr)) < 0) {
+#else /* INET6 */
 	    sizeof (sep->se_ctrladdr)) < 0) {
+#endif /* INET6 */
 		if (debug)
 			fprintf(stderr, "bind failed on %s/%s: %s\n",
 				sep->se_service, sep->se_proto,
@@ -877,6 +922,54 @@ more:
 	else
 		sep->se_socktype = -1;
 	sep->se_proto = newstr(sskip(&cp));
+
+#if INET6
+	{
+	  struct addrinfo req, *ai;
+	  int i;
+	  char *host, *serv;
+	  char buffer[strlen(sep->se_service) + 3];
+	  
+	  memset(&req, 0, sizeof(struct addrinfo));
+	  req.ai_flags = AI_PASSIVE;
+	  req.ai_socktype = sep->se_socktype;
+	  /* req.ai_proto = sep->se_proto; */
+	  
+	  if (serv = strrchr(sep->se_service, '.')) {
+	    *(serv++) = 0;
+	    host = sep->se_service;
+	  } else {
+	    serv = sep->se_service;
+	    host = NULL;
+	  };
+
+	  if (host && !strcmp(host, "*"))
+	    host = NULL;
+	  if (!serv || !strcmp(serv, "*")) {
+            syslog(LOG_ERR, "unspecified service isn't allowed (host=%s)", host ? host : "*");
+            if (debug)
+              fprintf(stderr, "unspecified service isn't allowed (host=%s)\n", host ? host : "*");
+            exit(1);
+          };
+
+	  sprintf(buffer, "%s.%s", host ? host : "*", serv);
+
+	  if ((i = getaddrinfo(host, serv, &req, &ai))) {
+	    syslog(LOG_ERR, "getaddrinfo %s: %s", sep->se_service, nrl_gai_strerror(i));
+            if (debug)
+	      fprintf(stderr, "getaddrinfo %s: %s\n", sep->se_service, nrl_gai_strerror(i));
+	    exit(1);
+	  };
+
+	  memcpy(&(sep->se_ctrladdr), ai->ai_addr, ai->ai_addrlen);
+
+	  freeaddrinfo(ai);
+
+	  free(sep->se_service);
+	  sep->se_service = newstr(buffer);
+	};
+#endif /* INET6 */
+
 	arg = sskip(&cp);
 	sep->se_wait = strcmp(arg, "wait") == 0;
 	if (ISMUX(sep)) {
@@ -900,14 +993,47 @@ more:
 		}
 	}
 	sep->se_user = newstr(sskip(&cp));
+
+#ifdef IPSEC
+	{
+	  char *c = newstr(sskip(&cp));
+
+	  sep->se_request = NULL;
+	  sep->se_requestlen = 0;
+
+	  if (strcmp(c, "*")) {
+	    if (nrl_strtoreq(c, &sep->se_request, &sep->se_requestlen)) {
+	      syslog(LOG_ERR, "nrl_strtoreq(%s) failed", c);
+	      exit(1);
+	    };
+	  };
+
+	  free(c);
+	};
+#endif /* IPSEC */
+
 	sep->se_server = newstr(sskip(&cp));
 	if (strcmp(sep->se_server, "internal") == 0) {
 		struct biltin *bi;
+#if INET6
+		char *c = strrchr(sep->se_service, '.');
+                if (!c) {
+                  syslog(LOG_ERR, "internal error -- '%s' missing service name", sep->se_service);
+                  if (debug)
+                    fprintf(stderr, "internal error -- '%s' missing service name", sep->se_service);
+                  exit(1);
+                };
+		c++;
 
+		for (bi = biltins; bi->bi_service; bi++)
+		  if (bi->bi_socktype == sep->se_socktype && !strcmp(bi->bi_service, c))
+                    break;
+#else /* INET6 */
 		for (bi = biltins; bi->bi_service; bi++)
 			if (bi->bi_socktype == sep->se_socktype &&
 			    strcmp(bi->bi_service, sep->se_service) == 0)
 				break;
+#endif /* INET6 */
 		if (bi->bi_service == 0) {
 			syslog(LOG_ERR, "internal service %s unknown",
 				sep->se_service);
@@ -1026,16 +1152,16 @@ inetd_setproctitle(a, s)
 	int size;
 	char *cp;
 #ifdef INET6
-	struct sockaddr_in6 sin;
+	su sa;
 #else /* INET6 */
 	struct sockaddr_in sin;
 #endif /* INET6 */
 	char buf[80];
 
 	cp = Argv[0];
-	size = sizeof(sin);
 #ifdef INET6
-	if (getpeername(s, (struct sockaddr *)&sin, &size) == 0) {
+	size = sizeof(su);
+	if (getpeername(s, (struct sockaddr *)&sa, &size) == 0) {
 	  int i;
 
 	  strcpy(buf, "-");
@@ -1044,13 +1170,14 @@ inetd_setproctitle(a, s)
 
 	  i = strlen(buf);
 
-	  if (nrl_getnameinfo(&sin, size, buf + i, sizeof(buf) - i, NULL, 0, nflag)) {
+	  if (nrl_getnameinfo((struct sockaddr *)&sa, size, buf + i, sizeof(buf) - i, NULL, 0, nflag)) {
 	    syslog(LOG_ERR, "getnameinfo failed");
 	    buf[i - 1] = 0;
 	  } else
 	    strcat(buf, "]");
 	}
 #else /* INET6 */
+	size = sizeof(sin);
 	if (getpeername(s, (struct sockaddr *)&sin, &size) == 0)
 		(void) sprintf(buf, "-%s [%s]", a, inet_ntoa(sin.sin_addr));
 #endif /* INET6 */
@@ -1314,11 +1441,30 @@ print_service(action, sep)
 	char *action;
 	struct servtab *sep;
 {
+#if IPSEC
+        char buffer[64];
+
+	if (sep->se_request) {
+	  if (nrl_reqtostr(sep->se_request, sep->se_requestlen, buffer, sizeof(buffer))) {
+	    fprintf(stderr, "nrl_reqtostr failed\n");
+	    exit(1);
+	  };
+	} else
+	  strcpy(buffer, "*");
+
+	fprintf(stderr,
+	    "%s: %s proto=%s, wait=%d, user=%s builtin=%x request=%s server=%s\n",
+	    action, sep->se_service, sep->se_proto,
+	    sep->se_wait, sep->se_user, (long)sep->se_bi,
+	    buffer,
+	    sep->se_server);
+#else /* IPSEC */
 	fprintf(stderr,
 	    "%s: %s proto=%s, wait=%d, user=%s builtin=%x server=%s\n",
 	    action, sep->se_service, sep->se_proto,
 	    sep->se_wait, sep->se_user, (int)sep->se_bi, 
 	    sep->se_server);
+#endif /* IPSEC */
 }
 
 /*
