@@ -21,8 +21,8 @@
  *  Optimization module for tcpdump intermediate representation.
  */
 #ifndef lint
-static char rcsid[] =
-    "@(#) $Header: /home/cmetz/src.CVS/inet6-apps/libpcap-0.2.1/optimize.c,v 1.1.1.1 1996/10/26 05:00:49 cmetz Exp $ (LBL)";
+static const char rcsid[] =
+    "@(#) $Header: /home/cmetz/src.CVS/inet6-apps/libpcap-0.3.1a2/optimize.c,v 1.1 1997/01/15 18:47:53 cmetz Exp $ (LBL)";
 #endif
 
 #include <sys/types.h>
