@@ -21,10 +21,6 @@
  * This code contributed by Atanu Ghosh (atanu@cs.ucl.ac.uk),
  * University College London.
  */
-#ifndef lint
-static  char rcsid[] =
-    "@(#)$Header: /home/cmetz/src.CVS/inet6-apps/libpcap-0.2.1/pcap-dlpi.c,v 1.1.1.1 1996/10/26 05:00:49 cmetz Exp $ (LBL)";
-#endif
 
 /*
  * Packet capture routine for dlpi under SunOS 5
@@ -40,6 +36,11 @@ static  char rcsid[] =
  *      kernel.
  */
 
+#ifndef lint
+static const char rcsid[] =
+    "@(#) $Header: /home/cmetz/src.CVS/inet6-apps/libpcap-0.3.1a2/pcap-dlpi.c,v 1.1 1997/01/15 18:47:53 cmetz Exp $ (LBL)";
+#endif
+
 #include <sys/types.h>
 #include <sys/time.h>
 #ifdef HAVE_SYS_BUFMOD_H
@@ -378,6 +379,7 @@ pcap_open_live(char *device, int snaplen, int promisc, int to_ms, char *ebuf)
 
 	case DL_FDDI:
 		p->linktype = DLT_FDDI;
+		p->offset = 3;
 		break;
 
 	default:
@@ -541,11 +543,18 @@ recv_ack(int fd, int size, const char *what, char *bufp, char *ebuf)
 			    what);
 			break;
 
+
 		case DL_SYSERR:
 			sprintf(ebuf, "recv_ack: %s: %s",
 			    what, pcap_strerror(dlp->error_ack.dl_unix_errno));
 			break;
 
+		case DL_UNSUPPORTED:
+			sprintf(ebuf,
+			    "recv_ack: %s: Service not supplied by provider",
+			    what);
+			break;
+
 		default:
 			sprintf(ebuf, "recv_ack: %s error 0x%x",
 			    what, (bpf_u_int32)dlp->error_ack.dl_errno);
@@ -593,6 +602,9 @@ dlbindreq(int fd, bpf_u_int32 sap, char *ebuf)
 	req.dl_service_mode = DL_HP_RAWDLS;
 #else
 	req.dl_sap = sap;
+#ifdef DL_CLDLS
+	req.dl_service_mode = DL_CLDLS;
+#endif
 #endif
 
 	return (send_request(fd, (char *)&req, sizeof(req), "bind", ebuf));
