@@ -123,9 +123,9 @@ typedef union {
 char addrbuf[sizeof(su)], addrbuf2[sizeof(su)];
 int addrlen;
 
-#if INET6 && defined(IPV6_PKTINFO)
+#if INET6 && defined(IPV6_RXINFO)
 struct in6_pktinfo in6_pktinfo;
-#endif /* INET6 && defined(IPV6_PKTINFO) */
+#endif /* INET6 && defined(IPV6_RXINFO) */
 
 char controlbuf[1024];
 
@@ -293,7 +293,7 @@ main(argc, argv)
   if (!portname)
     portname = "7777";
 
-#if INET6 && defined(IPV6_PKTINFO)
+#if INET6 && defined(IPV6_TXINFO)
   if (optind < argc)
     if (!(in6_pktinfo.ipi6_ifindex = if_nametoindex(argv[optind]))) {
       fprintf(stderr, "%s: %s: unknown/invalid interface name\n", myname, argv[optind]);
@@ -311,7 +311,7 @@ main(argc, argv)
     };
     in6_pktinfo.ipi6_addr = ((struct sockaddr_in6 *)(ai->ai_addr))->sin6_addr;
   };
-#endif /* INET6 && defined(IPV6_PKTINFO) */
+#endif /* INET6 && defined(IPV6_TXINFO) */
 
   memset(&req, 0, sizeof(struct addrinfo));
 
@@ -445,15 +445,15 @@ main(argc, argv)
     addrform(aform);
 #endif /* INET6 && defined(IPV6_ADDRFORM) */
 
-#if INET6 && defined(IPV6_PKTINFO)
+#if INET6 && defined(IPV6_RXINFO)
   if (af == AF_INET6) {
     int val = 1;
-    if (setsockopt(s, IPPROTO_IPV6, IPV6_PKTINFO, &val, sizeof(val)) < 0) {
-      perror("setsockopt (IPV6_PKTINFO)");
+    if (setsockopt(s, IPPROTO_IPV6, IPV6_RXINFO, &val, sizeof(val)) < 0) {
+      perror("setsockopt (IPV6_RXINFO)");
       exit(1);
     }
   };
-#endif /* INET6 && defined(IPV6_PKTINFO) */
+#endif /* INET6 && defined(IPV6_RXINFO) */
 
   if (nrl_getnameinfo(ai->ai_addr, ai->ai_addrlen, hbuf, sizeof(hbuf), sbuf, sizeof(sbuf), nflag)) {
     fprintf(stderr, "getnameinfo() failed!\n");
@@ -493,7 +493,7 @@ main(argc, argv)
 
     ((struct cmsghdr *)controlbuf)->cmsg_len = sizeof(struct cmsghdr) + sizeof(struct in6_pktinfo);
     ((struct cmsghdr *)controlbuf)->cmsg_level = IPPROTO_IPV6;
-    ((struct cmsghdr *)controlbuf)->cmsg_type = IPV6_PKTINFO;
+    ((struct cmsghdr *)controlbuf)->cmsg_type = IPV6_RXINFO;
 
     memcpy(controlbuf + sizeof(struct cmsghdr), &in6_pktinfo, sizeof(struct in6_pktinfo));
   } else {
@@ -625,11 +625,11 @@ servercode:
       };
     };
 
-#if INET6 && defined(IPV6_PKTINFO)
+#if INET6 && defined(IPV6_RXINFO)
     {
       struct cmsghdr *cmsghdr;
       for (cmsghdr = CMSG_FIRSTHDR(&msghdr2); cmsghdr; cmsghdr = CMSG_NXTHDR(&msghdr2, cmsghdr))
-	if ((cmsghdr->cmsg_level == IPPROTO_IPV6) && (cmsghdr->cmsg_type == IPV6_PKTINFO))
+	if ((cmsghdr->cmsg_level == IPPROTO_IPV6) && (cmsghdr->cmsg_type == IPV6_RXINFO))
 	  if (memcmp(CMSG_DATA(cmsghdr), &in6_pktinfo, sizeof(struct in6_pktinfo))) {
 	    struct sockaddr_in6 sin6;
 	    memcpy(&in6_pktinfo, CMSG_DATA(cmsghdr), sizeof(struct in6_pktinfo));
@@ -649,7 +649,7 @@ servercode:
 	    };
 	  };
     };
-#endif /* INET6 && defined(IPV6_PKTINFO) */
+#endif /* INET6 && defined(IPV6_RXINFO) */
 
     if (datasize) {
       datablock[datasize] = 0;
