@@ -1506,20 +1506,16 @@ pr_retip(ip)
 	  do {
 	    cp += size;
 	    switch(proto) {
-#ifdef IPPROTO_HOP
+#if 0
 		case IPPROTO_HOP:
 		  /* punt for now*/
 		  size = ((struct ipv6_opthdr *)cp)->oh_nexthdr + 8;
 		  proto = ((struct ipv6_opthdr *)cp)->oh_nexthdr;
 		  break;
-#endif /* IPPROTO_HOP */
-#if IPSEC
 		case IPPROTO_AH:
 		  size = ((struct ipsec_ah *)cp)->ah_datalen * 8 + 8;
 		  proto = ((struct ipsec_ah *)cp)->ah_nexthdr;
 		  break;
-#endif /* IPSEC */
-#ifdef IPPROTO_ROUTING
 		case IPPROTO_ROUTING:
 		  if (((struct ipv6_srcroute0 *)cp)->i6sr_type == 0) {
 		    size = ((struct ipv6_srcroute0 *)cp)->i6sr_numaddrs *
@@ -1530,13 +1526,11 @@ pr_retip(ip)
 		    return;
 		  }
 		  break;	      
-#endif /* IPPROTO_ROUTING */
-#ifdef IPPROTO_FRAGMENT
 		case IPPROTO_FRAGMENT:
 		  size = 8;
 		  proto = ((struct ipv6_fraghdr *)cp)->frag_nexthdr;
 		  break;
-#endif /* IPPROTO_FRAGMENT */
+#endif /* 0 */
 		default:
 		  size = -1;
 		  break;	    
