@@ -27,8 +27,8 @@
  *  and address to string conversion routines
  */
 #ifndef lint
-static char rcsid[] =
-    "@(#) $Header: /home/cmetz/src.CVS/inet6-apps/tcpdump-3.2.1/addrtoname.c,v 1.1.1.1 1996/10/26 05:00:50 cmetz Exp $ (LBL)";
+static const char rcsid[] =
+    "@(#) $Header: /home/cmetz/src.CVS/inet6-apps/tcpdump-3.3.1a2/addrtoname.c,v 1.1 1997/01/15 18:49:05 cmetz Exp $ (LBL)";
 #endif
 
 #include <sys/types.h>
@@ -185,7 +185,7 @@ getname(const u_char *ap)
 	addr = *(const u_int32_t *)ap;
 #else
 	/*
-	 * Deal with alignment.
+	 * Extract 32 bits in network order, dealing with alignment.
 	 */
 	switch ((long)ap & 3) {
 
@@ -194,26 +194,26 @@ getname(const u_char *ap)
 		break;
 
 	case 2:
-#if BYTE_ORDER == LITTLE_ENDIAN
-		addr = ((u_int32_t)*(u_short *)(ap + 2) << 16) |
-			(u_int32_t)*(u_short *)ap;
-#else
+#ifdef WORDS_BIGENDIAN
 		addr = ((u_int32_t)*(u_short *)ap << 16) |
 			(u_int32_t)*(u_short *)(ap + 2);
+#else
+		addr = ((u_int32_t)*(u_short *)(ap + 2) << 16) |
+			(u_int32_t)*(u_short *)ap;
 #endif
 		break;
 
 	default:
-#if BYTE_ORDER == LITTLE_ENDIAN
-		addr = ((u_int32_t)ap[3] << 24) |
-			((u_int32_t)ap[2] << 16) |
-			((u_int32_t)ap[1] << 8) |
-			(u_int32_t)ap[0];
-#else
+#ifdef WORDS_BIGENDIAN
 		addr = ((u_int32_t)ap[0] << 24) |
 			((u_int32_t)ap[1] << 16) |
 			((u_int32_t)ap[2] << 8) |
 			(u_int32_t)ap[3];
+#else
+		addr = ((u_int32_t)ap[3] << 24) |
+			((u_int32_t)ap[2] << 16) |
+			((u_int32_t)ap[1] << 8) |
+			(u_int32_t)ap[0];
 #endif
 		break;
 	}
@@ -229,14 +229,16 @@ getname(const u_char *ap)
 	/*
 	 * Only print names when:
 	 *	(1) -n was not given.
-	 *	(2) Address is foreign and -f was given.  If -f was not
-	 *	    present, f_netmask and f_local are 0 and the second
-	 *	    test will succeed.
-	 *	(3) The host portion is not 0 (i.e., a network address).
-	 *	(4) The host portion is not broadcast.
+	 *      (2) Address is foreign and -f was given. (If -f was not
+	 *	    give, f_netmask and f_local are 0 and the test
+	 *	    evaluates to true)
+	 *      (3) -a was given or the host portion is not all ones
+	 *          nor all zeros (i.e. not a network or broadcast address)
 	 */
-	if (!nflag && (addr & f_netmask) == f_localnet
-	    && (addr &~ netmask) != 0 && (addr | netmask) != 0xffffffff) {
+	if (!nflag &&
+	    (addr & f_netmask) == f_localnet &&
+	    (aflag ||
+	    !((addr & ~netmask) == 0 || (addr | netmask) == 0xffffffff))) {
 		if (!setjmp(getname_env)) {
 			(void)signal(SIGALRM, nohostname);
 			(void)alarm(20);
@@ -787,7 +789,7 @@ init_llcsaparray(void)
  * of the local network.  mask is its subnet mask.
  */
 void
-init_addrtoname(int fflag, u_int32_t localnet, u_int32_t mask)
+init_addrtoname(u_int32_t localnet, u_int32_t mask)
 {
 	netmask = mask;
 	if (fflag) {
@@ -829,7 +831,7 @@ dnaddr_string(u_short dnaddr)
 
 /* Return a zero'ed hnamemem struct and cuts down on calloc() overhead */
 struct hnamemem *
-newhnamemem()
+newhnamemem(void)
 {
 	register struct hnamemem *p;
 	static struct hnamemem *ptr = NULL;
