@@ -20,8 +20,8 @@
  */
 
 #ifndef lint
-static char rcsid[] =
-    "@(#) $Header: /home/cmetz/src.CVS/inet6-apps/tcpdump-3.2.1/print-decnet.c,v 1.1.1.1 1996/10/26 05:00:51 cmetz Exp $ (LBL)";
+static const char rcsid[] =
+    "@(#) $Header: /home/cmetz/src.CVS/inet6-apps/tcpdump-3.3.1a2/print-decnet.c,v 1.1 1997/01/15 18:49:11 cmetz Exp $ (LBL)";
 #endif
 
 #include <sys/param.h>
@@ -733,7 +733,7 @@ char *
 dnnum_string(u_short dnaddr)
 {
 	char *str;
-	int area = (dnaddr & AREAMASK) >> AREASHIFT;
+	int area = (u_short)(dnaddr & AREAMASK) >> AREASHIFT;
 	int node = dnaddr & NODEMASK;
 
 	str = (char *)malloc(sizeof("00.0000"));
