@@ -117,9 +117,10 @@ Research Laboratory (NRL).
 #include <netinet/tcp.h>
 #include <netinet/tcpip.h>
 
-#include <netinet6/ipv6.h>
 #include <netinet6/in6.h>
-#include <netinet6/ipv6_icmp.h>
+#include <netinet6/ipv6.h>
+#include <netinet6/icmpv6.h>
+#include <netinet6/nd6_protocol.h>
 #include <stdio.h>
 
 #include <string.h>
@@ -181,10 +182,10 @@ void na_opt_print(struct na_opt *na, char *s)
 void
 icmpv6_print(register const u_char *bp, int length)
 {
-	register const struct ipv6_icmp *dp;
+	register const struct icmpv6hdr *icmpv6;
 	register char *str;
 	register const struct ipv6 *oip;
-	register const struct udphdr *ouh;
+	register const struct uicmpv6hdr *ouh;
 	char buf[256];
 	char src[80];
 	char dst[80];
@@ -195,21 +196,21 @@ icmpv6_print(register const u_char *bp, int length)
 	char *targ_addr;
 	int na_len, left;
 
-	dp = (struct ipv6_icmp *)bp;
+	icmpv6 = (struct icmpv6hdr *)bp;
 	left = (u_int)snapend - (u_int)bp;
 
 	str = buf;
 	*str = '\0';
 
-	if (left < ICMPV6_MINLEN)
+	if (left < sizeof(struct icmpv6hdr))
 	  goto trunc;
 
-	switch (dp->icmp_type) {
-	case ICMPV6_ECHOREPLY:
+	switch (icmpv6->icmpv6_type) {
+	case ICMPV6_ECHO_REPLY:
 		str = "echo reply";
 		break;
-	case ICMPV6_UNREACH:
-		switch (dp->icmp_code) {
+	case ICMPV6_DST_UNREACH:
+		switch (icmpv6->icmpv6_code) {
 		case ICMPV6_UNREACH_NOROUTE:
 			str = "no route";
 			break;
@@ -230,53 +231,61 @@ icmpv6_print(register const u_char *bp, int length)
 			break;
 		}
 		break;
-	case ICMPV6_TOOBIG:
-		sprintf(str, "packet too big: mtu[%d]",
-                                 ntohl(dp->icmp_nexthopmtu));
+	case ICMPV6_PACKET_TOOBIG:
+		sprintf(str, "packet too big: mtu[%d]", ntohl(icmpv6->icmpv6_mtu));
 		break;
-	case ICMPV6_ECHO:
+	case ICMPV6_ECHO_REQUEST:
 		str = "echo request";
 		break;
-	case ICMPV6_GRPQUERY:
+	case ICMPV6_MEMBERSHIP_QUERY:
 		str = "query group membership";
 		break;
-	case ICMPV6_GRPREPORT:
+	case ICMPV6_MEMBERSHIP_REPORT:
 		str = "join mcast group";
 		break;
-	case ICMPV6_GRPTERM:
+	case ICMPV6_MEMBERSHIP_REDUCTION:
 		str = "leave mcast group";
 		break;
-	case ICMPV6_TIMXCEED:
-		switch (dp->icmp_code) {
-		case ICMPV6_TIMXCEED_INTRANS:
-			str = "time exceeded in-transit";
+	case ICMPV6_TIME_EXCEEDED:
+		switch (icmpv6->icmpv6_code) {
+		case ICMPV6_EXCEEDED_HOPS:
+			str = "hop limit exceeded";
 			break;
-		case ICMPV6_TIMXCEED_REASS:
-			str = "ip reassembly time exceeded";
+		case ICMPV6_EXCEEDED_REASSEMBLY:
+			str = "reassembly time exceeded";
 			break;
 		}
 		break;
-	case ICMPV6_PARAMPROB:
-		switch(dp->icmp_code) {
-		case ICMPV6_PARAMPROB_PROB:
-		  strcpy(str,"incorrect parameter");
+	case ICMPV6_PARAMETER_PROBLEM:
+		switch(icmpv6->icmpv6_code) {
+		case ICMPV6_PARAMPROB_HDR:
+		  strcpy(str, "bad header field");
 		  break;
 		case ICMPV6_PARAMPROB_NEXTHDR:
 		  strcpy(str,"bad next header");
 		  break;
-		case ICMPV6_PARAMPROB_BADOPT:
+		case ICMPV6_PARAMPROB_OPTION:
 		  strcpy(str,"unrec. option");
 		  break;
 		}
-		sprintf(str+strlen(str), " ptr[%x]", dp->icmp_paramptr);
+		sprintf(str+strlen(str), " ptr[%x]", icmpv6->icmpv6_pptr);
 		break;
-	case ICMPV6_ROUTERSOL:
+#ifndef ICMPV6_ROUTER_SOLICITATION
+#define ICMPV6_ROUTER_SOLICITATION ND6_ROUTER_SOLICITATION
+#endif /* ICMPV6_ROUTER_SOLICITATION */
+	case ICMPV6_ROUTER_SOLICITATION:
 		str = "router solicit";
 		break;
-	case ICMPV6_ROUTERADV:
+#ifndef ICMPV6_ROUTER_ADVERTISEMENT
+#define ICMPV6_ROUTER_ADVERTISEMENT ND6_ROUTER_ADVERTISEMENT
+#endif /* ICMPV6_ROUTER_ADVERTISEMENT */
+	case ICMPV6_ROUTER_ADVERTISEMENT:
 		str = "router ad";
 		break;
-	case ICMPV6_NEIGHBORSOL:
+#ifndef ICMPV6_NEIGHBOR_SOLICITATION
+#define ICMPV6_NEIGHBOR_SOLICITATION ND6_NEIGHBOR_SOLICITATION
+#endif /* ICMPV6_NEIGHBOR_SOLICITATION */
+	case ICMPV6_NEIGHBOR_SOLICITATION:
 		if (left < sizeof(struct nbr_sol))
 		  goto trunc;
                 str = strcat(str, "neigh sol ");
@@ -298,7 +307,10 @@ icmpv6_print(register const u_char *bp, int length)
                         opt = (struct na_opt *)cp;
 		      }
 		break;
-	case ICMPV6_NEIGHBORADV:
+#ifndef ICMPV6_NEIGHBOR_ADVERTISEMENT
+#define ICMPV6_NEIGHBOR_ADVERTISEMENT ND6_NEIGHBOR_ADVERTISEMENT
+#endif /* ICMPV6_NEIGHBOR_ADVERTISEMENT */
+	case ICMPV6_NEIGHBOR_ADVERTISEMENT:
 		if (left < sizeof(struct nbr_ad))
 		  goto trunc;
 		str = strcat(str, "neigh adv ");
@@ -309,13 +321,19 @@ icmpv6_print(register const u_char *bp, int length)
 		 * Flags are in network order, but so are bitmasks.
 		 * (See netinet6/ipv6_icmp.h for details.)
 		 */
+		if (na->flags & ND6_NADVERFLAG_ISROUTER)
+		  str = strcat(str, "R");
+		if (na->flags & ND6_NADVERFLAG_SOLICITED)
+		  str = strcat(str, "S");
+		if (na->flags & ND6_NADVERFLAG_OVERRIDE)
+		  str = strcat(str, "O");
+#if 0
 		if (na->flags & ICMPV6_NEIGHBORADV_RTR)
 		  str = strcat(str, "R");
 		if (na->flags & ICMPV6_NEIGHBORADV_SOL)
 		  str = strcat(str, "S");
 		if (na->flags & ICMPV6_NEIGHBORADV_OVERRIDE)
 		  str = strcat(str, "O");
-#if 0
 		if (na->flags & ICMPV6_NEIGHBORADV_SECOND)
 		  str = strcat(str, "N");
 #endif /* 0 */
@@ -336,7 +354,7 @@ icmpv6_print(register const u_char *bp, int length)
 		}
 		break;
 	default:
-		(void)sprintf(buf, "type#%d", dp->icmp_type);
+		(void)sprintf(buf, "type#%d", icmpv6->icmpv6_type);
 		break;
 	}
         (void)printf("icmpv6: %s", str);
