@@ -140,7 +140,32 @@ static void hop_print(const u_char * bp)
 	printf(" hop");
 }
 
-static void frag_print(struct ipv6_fraghdr *ip)
+struct __ipv6_fraghdr
+{
+  u_int8_t frag_nextheader;
+  u_int8_t frag_reserved;
+  u_int16_t frag_bitsoffset;
+  u_int32_t frag_id;
+};
+
+struct __ipv6_opthdr
+{
+  u_int8_t oh_nextheader;
+  u_int8_t oh_extlen;
+  u_int8_t oh_data[6];
+};
+
+struct __ipv6_srcroute0
+{
+  u_int8_t i6sr_nextheader;
+  u_int8_t i6sr_numaddrs;
+  u_int8_t i6sr_type;
+  u_int8_t i6sr_left;
+  u_int8_t i6sr_reserved;
+  u_int8_t i6sr_mask[3];
+};
+
+static void frag_print(struct __ipv6_fraghdr *ip)
 {
   u_int16_t bitoffset;
 
@@ -151,7 +176,7 @@ static void frag_print(struct ipv6_fraghdr *ip)
   printf(" %u] ", ntohl(ip->frag_id));
 }
 
-static void route_print(struct ipv6_srcroute0 *ip)
+static void route_print(struct __ipv6_srcroute0 *ip)
 {
 	printf(" %d", ip->i6sr_type);
 	if (!&ip->i6sr_type)  {
@@ -159,48 +184,47 @@ static void route_print(struct ipv6_srcroute0 *ip)
 	}
 }
 
-static void dest_print(struct ipv6_opthdr *ip)
+static void dest_print(struct __ipv6_opthdr *ip)
 {
-	printf(" %d", ip-> oh_extlen);
+	printf(" %d", ip->oh_extlen);
 }
 
 void ipv6_print (register const u_char *bp, register int length)
 {
 	char buf[80];
-	register const struct ipv6 *ip;
+	register const struct ipv6hdr *ip;
 	u_int8_t nhdr;
 	int first = 1, len=0;
 	char *cp;
 	u_int32_t p;	
 	struct hostent *name;
 
-	ip= (const struct ipv6 *) bp;
-	if (length < sizeof(struct ipv6)){
+	ip= (const struct ipv6hdr *) bp;
+	if (length < sizeof(struct ipv6hdr)){
 	  (void)printf("truncated-ipv6 %d", length);
 	  return;
 	}
 	
-	len = sizeof(struct ipv6) + ntohs(ip->ipv6_length);
+	len = sizeof(struct ipv6hdr) + ntohs(ip->ipv6_len);
 
 	if (length < len)
 	  (void)printf("truncated-ipv6 - %d bytes missing!",
 		       len - length);
 
-	len = sizeof(struct ipv6);
+	len = sizeof(struct ipv6hdr);
 
         printf(" %s > %s ", ipv6addr_string(&(ip->ipv6_src)), 
 	       ipv6addr_string(&(ip->ipv6_dst)));
 
 	if (vflag) {
-	  p = ntohl(ip->ipv6_versfl);
-	  printf(" (v%d,", p>>28);                     /* version */
+	  printf(" (v%d,", ip->ipv6_version);                     /* version */
 	  printf(" priority %d,", (p&0x0f000000)>>24); /* priority */
 	  printf(" flow %d,", p&0x00ffffff);	       /* flow label */
-	  printf(" len %d,", ntohs(ip->ipv6_length));  /* payload length */
+	  printf(" len %d,", ntohs(ip->ipv6_len));  /* payload length */
 	  printf(" hop %d) ", ip->ipv6_hoplimit);
 	}
 	
-	nhdr = ip->ipv6_nexthdr;
+	nhdr = ip->ipv6_nextheader;
 	cp = (char *)(ip + 1);
 
 	while ((nhdr!=IPPROTO_NONE)&&(nhdr!=IPPROTO_TCP)&&(nhdr!=IPPROTO_UDP)
@@ -217,21 +241,21 @@ void ipv6_print (register const u_char *bp, register int length)
 			break;
 
 		case IPPROTO_ROUTING:	/* routing header (43)*/
-			len += sizeof (struct ipv6_srcroute0 );
+			len += sizeof(struct __ipv6_srcroute0 );
 			if ((len<snaplen)&&(vflag))
-				route_print((struct ipv6_srcroute0 *)cp);
+				route_print((struct __ipv6_srcroute0 *)cp);
 			else
 				printf(" route-hdr");
-			nhdr = ((struct ipv6_srcroute0 *)cp)->i6sr_nexthdr;
+			nhdr = ((struct __ipv6_srcroute0 *)cp)->i6sr_nextheader;
 			break;
 
 		case IPPROTO_FRAGMENT:	/* fragment header (44)*/
-			len += sizeof (struct ipv6_fraghdr);
+			len += sizeof(struct __ipv6_fraghdr);
 			if ((len<snaplen)&&(vflag))
-				frag_print((struct ipv6_fraghdr *)cp);
+				frag_print((struct __ipv6_fraghdr *)cp);
 			else
 				printf(" frag ");
-			nhdr = ((struct ipv6_fraghdr *) cp)->frag_nexthdr;
+			nhdr = ((struct __ipv6_fraghdr *)cp)->frag_nextheader;
 			printf("nhdr=");
 			switch (nhdr)  {
 			case IPPROTO_HOP:
@@ -299,13 +323,13 @@ void ipv6_print (register const u_char *bp, register int length)
 #endif /* IPSEC */
 
 		case IPPROTO_DST:	/* destination options header (60)*/
-			len += sizeof (struct ipv6_opthdr);
+			len += sizeof(struct __ipv6_opthdr);
 			if ((len<snaplen)&&(vflag))
-				dest_print((struct ipv6_opthdr *)cp);
+				dest_print((struct __ipv6_opthdr *)cp);
 			else
 				printf(" dest-hdr");
-			nhdr = ((struct ipv6_opthdr *)cp)->oh_nexthdr;
-			cp += sizeof(struct ipv6_opthdr);
+			nhdr = ((struct __ipv6_opthdr *)cp)->oh_nextheader;
+			cp += sizeof(struct __ipv6_opthdr);
 			break;
 
 		default:
