@@ -26,7 +26,7 @@
 
 #ifndef lint
 static const char rcsid[] =
-    "@(#) $Header: /home/cmetz/src.CVS/inet6-apps/tcpdump-3.3.1a2/print-tcp.c,v 1.1 1997/01/15 18:49:16 cmetz Exp $ (LBL)";
+    "@(#) $Header: /home/cmetz/src.CVS/inet6-apps/tcpdump-3.3.1a2/print-tcp.c,v 1.2 1997/01/18 17:48:52 cmetz Exp $ (LBL)";
 #endif
 
 #include <sys/param.h>
@@ -134,8 +134,8 @@ tcp_print(register const u_char *bp, register u_int length,
 #ifdef INET6
 	if (ip->ip_v == 6)
 	  printf("%s.%s > %s.%s:",
-		ipv6addr_string(&(((struct ipv6 *)ip)->ipv6_src)), tcpport_string(sport),
-		ipv6addr_string(&(((struct ipv6 *)ip)->ipv6_dst)), tcpport_string(dport));
+		ipv6addr_string(&(((struct ipv6hdr *)ip)->ipv6_src)), tcpport_string(sport),
+		ipv6addr_string(&(((struct ipv6hdr *)ip)->ipv6_dst)), tcpport_string(dport));
 	else
 #endif /* INET6 */
 	(void)printf("%s.%s > %s.%s: ",
