@@ -84,10 +84,6 @@ static char sccsid[] = "@(#)commands.c	8.4 (Berkeley) 5/30/95";
 #include <setjmp.h>
 #endif /* FASTCTO */
 
-#ifdef INET6
-#include "support.h"
-#endif /* INET6 */
-
 #ifndef	MAXHOSTNAMELEN
 #define	MAXHOSTNAMELEN 64
 #endif	MAXHOSTNAMELEN
@@ -2111,7 +2107,7 @@ tn(argc, argv)
     unsigned long sourceroute(), srlen;
 #endif
     char *cmd, *hostp = 0, *portp = 0, *user = 0;
-#ifdef INET6
+#if INET6
     struct addrinfo req, *ai, *ai2;
     char buf1[46], buf2[16];
     int i;
